/*
 * SSD1306_I2C.h
 *
 * Created: 2017-04-01
 * version: 2017-04-01 
 *  Author: uwezi
 */ 
#ifndef SSD1306_I2C_H_
#define SSD1306_I2C_H_

#include <avr/pgmspace.h>

#define SSD1306_DEFAULT_ADDRESS 0x78  // device address of SSD1306 OLED
#define SSD1306_SETCONTRAST 0x81
#define SSD1306_DISPLAYALLON_RESUME 0xA4
#define SSD1306_DISPLAYALLON 0xA5
#define SSD1306_NORMALDISPLAY 0xA6
#define SSD1306_INVERTDISPLAY 0xA7
#define SSD1306_DISPLAYOFF 0xAE
#define SSD1306_DISPLAYON 0xAF
#define SSD1306_SETDISPLAYOFFSET 0xD3
#define SSD1306_SETCOMPINS 0xDA
#define SSD1306_SETVCOMDETECT 0xDB
#define SSD1306_SETDISPLAYCLOCKDIV 0xD5
#define SSD1306_SETPRECHARGE 0xD9
#define SSD1306_SETMULTIPLEX 0xA8
#define SSD1306_SETLOWCOLUMN 0x00
#define SSD1306_SETHIGHCOLUMN 0x10
#define SSD1306_SETSTARTLINE 0x40
#define SSD1306_MEMORYMODE 0x20
#define SSD1306_COLUMNADDR 0x21
#define SSD1306_PAGEADDR   0x22
#define SSD1306_COMSCANINC 0xC0
#define SSD1306_COMSCANDEC 0xC8
#define SSD1306_SEGREMAP 0xA0
#define SSD1306_CHARGEPUMP 0x8D
#define SSD1306_EXTERNALVCC 0x01
#define SSD1306_SWITCHCAPVCC 0x02
#define SSD1306_NOP 0xE3

#define SSD1306_RIGHT_HORIZONTAL_SCROLL 0x26              ///< Init rt scroll
#define SSD1306_LEFT_HORIZONTAL_SCROLL 0x27               ///< Init left scroll
#define SSD1306_VERTICAL_AND_RIGHT_HORIZONTAL_SCROLL 0x29 ///< Init diag scroll
#define SSD1306_VERTICAL_AND_LEFT_HORIZONTAL_SCROLL 0x2A  ///< Init diag scroll
#define SSD1306_DEACTIVATE_SCROLL 0x2E                    ///< Stop scroll
#define SSD1306_ACTIVATE_SCROLL 0x2F                      ///< Start scroll
#define SSD1306_SET_VERTICAL_SCROLL_AREA 0xA3             ///< Set scroll range

#define WIRE_MAX 32


//#define DISPLAYROTATE

#define DISPLAYSIZEX 128
#define DISPLAYSIZEY 32
// use charge pump 0x14, use Vcc 0x10
#define VCCSTATE 0x14

extern uint8_t framebuffer[DISPLAYSIZEX*DISPLAYSIZEY/8];


/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_clearbuffer
  Description  :  Clears the framebuffer but does not transfer to LCD.
  Argument(s)  :  None.
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_clearbuffer(void);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_update
  Description  :  transfers the local copy to the display
  Argument(s)  :  None.
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_update (void);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_clear
  Description  :  Clears the display
  Argument(s)  :  None.
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_clear ( void );

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_init
  Description  :  LCD controller initialization.
  Argument(s)  :  contrast value VOP
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_init ( void );

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_setpixel
  Description  :  Sets the pixel at xy location
  Argument(s)  :  x - range: 0 to DISPLAYSIZEX
                  y - range: 0 to DISPLAYSIZEY
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_setpixel(uint16_t x, uint16_t y);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_clearpixel
  Description  :  Clears the pixel at xy location
  Argument(s)  :  x - range: 0 to DISPLAYSIZEX
                  y - range: 0 to DISPLAYSIZEY
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_clearpixel(uint16_t x, uint16_t y);
extern void DISPLAY_drawpixel(uint16_t x, uint16_t y, uint8_t color);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_hLine
  Description  :  draw a horizontal line
  Argument(s)  :  x1, x2, y, color
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_hLine(uint16_t x1, uint16_t x2, uint16_t y, uint8_t color);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_Rect
  Description  :  draw a rectangle
  Argument(s)  :  x1, y1, w, h, color
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_Rect(uint16_t x1, uint16_t y1, uint16_t w, uint16_t h, uint8_t color);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_vLine
  Description  :  draw a vertical line
  Argument(s)  :  x, y1, y2, color
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_vLine(uint16_t x, uint16_t y1, uint16_t y2, uint8_t color);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_putchar
  Description  :  puts a single character onto LCD
  Argument(s)  :  x - range: 0 to DISPLAYSIZEX
                  y - range: 0 to DISPLAYSIZEY
                  ch - character
                  attr - attribute 0-normal, 1-inverse, 2-underline
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_putchar(uint8_t x0, uint8_t y0, char ch, uint8_t attr);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_print
  Description  :  prints a string
  Argument(s)  :  x - range: 0 to DISPLAYSIZEX
                  y - range: 0 to DISPLAYSIZEY
                  *ch - pointer t string
                  attr - attribute 0-normal, 1-inverse, 2-underline
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
extern void DISPLAY_print(uint8_t x, uint8_t y, char *ch, uint8_t attr);

// _p version transferring a string from flash memory
extern void DISPLAY_print_p(uint8_t x, uint8_t y, const char *ch, uint8_t attr);

// Macro to automatically put a string constant into program memory
#define DISPLAY_print_P(x, y, __s, attr)    DISPLAY_print_p(x, y, PSTR(__s), attr);

/*--------------------------------------------------------------------------------------------------
  Name         :  DISPLAY_scroll
  Description  :  softscrolls the framebuffer
  Argument(s)  :  dy - range: 0 to +/-DISPLAYSIZEY
  Return value :  None.
--------------------------------------------------------------------------------------------------*/
void DISPLAY_scroll(int8_t dy);

#endif /* SSD1306_I2C_H_ */
